module Hermes
  class HousingRequestInitializer
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::SopraXmlSender
    RANDOM_LIST_CHAR = '234567890123456789012345678901234567890123456789123abcdefghijkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ'
    
    field :unique_number, type: String
    field :hashed_pin_code, type: String
    field :pin_code_created_at, type: DateTime
    field :email, type: String
    field :xml_res, type: String
    
    field :done, type: Boolean

    common_indexes *%i[
      unique_number
      hashed_pin_code
      pin_code_created_at
      email
      done
    ]

    
    belongs_to(
      :housing_request,
      class_name: 'Hermes::HousingRequest',
      index: true
    )
    
    def self.get_all_done
      Hermes::HousingRequestInitializer.where({done: true}).each do |hri|
        ap hri.unique_number
      end
    end
    
    def self.get_all
      Hermes::HousingRequestInitializer.all.each do |hri|
        ap hri.unique_number
      end
    end

    def hidden_email
      return nil unless email

      splited = email.split('@')
      char_count = ((splited[0].length) - 2)
      splited[0] = "#{splited[0][0]}#{'*'*char_count}#{splited[0][(char_count + 1)]}"
      splited.join('@')
    end
    
    def hash_string(str)
      Digest::SHA2.new(512).hexdigest(str)
    end
    
    def format_mail_body(pin_code)
      "Bonjour,

      vous recevez ce mail suite à une tentative de récupération de votre demande sur la base de son numéro unique.
      Afin de confirmer cette récupération pour que votre demande soit dupliquée sur notre site Al'In, veuillez saisir la clé de sécurité dans la zone prévue à cet effet.

      Clé de sécurité : #{pin_code}

      Attention, cette clé de sécurité est valable 24 heures. Passé ce délai il sera nécessaire de recommencer votre démarche.
      Si vous n'êtes pas à l'origine de cette demande, veuillez ne pas prendre en considération ce mail.

      Cordialement

      Toute l'équipe d'Al'in

      ps : ceci est un message automatique, il est inutile de répondre à ce mail.
      "
    end
    
    def call_sne
      if Rails.env.test?
        path = [Hermes::Engine.root, %w[spec xml_bouchon], "DEM_" + unique_number + ".xml"].flatten.join('/').to_s
        pn = Pathname.new(path)
        return {err: 'unknown file'} unless pn.exist?
        self.xml_res = File.read(pn)
      else
        so = Hermes::BoClient::FindNudTask.new(housing_request_initializer_id: id)
        self.xml_res = ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml, "post", "Hermes::HousingRequestInitializer", id.to_s) || nil
        if !self.xml_res
          return {email: false, err: "SNE request failed"}
        end
      end

      parsed_xml = parse_xml(xml_res)
      user_mail = parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["demandeur"]["mel"]

      {email: user_mail, err: nil}
    end
    
    def send_pin_code_on_mail
      sne_response = call_sne
      return false if sne_response[:err]
      return :email_empty unless sne_response[:email]

      self.set(email: sne_response[:email], updated_at: DateTime.current)

      random_list_array = RANDOM_LIST_CHAR.split(//)
      pin_code = ''

      6.times do |i|
        pin_code += random_list_array[(SecureRandom.random_number * random_list_array.length).floor]
      end

      self.hashed_pin_code = hash_string(pin_code)
      self.pin_code_created_at = DateTime.current
      mail = Agent.bulk_mail(
        sne_response[:email],
        'securite@masolutionlogement.fr',
        'Récupération demande SNE',
        format_mail_body(pin_code)
      )
      return false unless mail.deliver_later

      save
      pin_code
    end
    
    def check_pin_code(user_input)
      return false unless hashed_pin_code && pin_code_created_at
      
      hash_string(user_input) == hashed_pin_code && (Time.current - pin_code_created_at) < 24.hours
    end
    
    def cant_create_new
      self.done && housing_request && housing_request.request_status != "ANNULEE"
    end 
  
    def hydrate_housing_request_from_sne(user_id)
      parsed_xml = parse_xml(xml_res)
      
      main_applicant = Hermes::Applicant.find_or_create_by(user_id: user_id, main_user_id: user_id)
      housing_request = Hermes::HousingRequest.create(user_id: user_id, main_user_id: user_id, main_applicant: main_applicant.id)
      data = {unique_number: unique_number, housing_request_id: housing_request.id, creation_date: parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["dateCreationDemande"]}
      
      nud = Hermes::Nud.create(data)

      housing_request.assign_attributes(Hermes::BoClient::Mappings::HousingRequestReverseMap.new(parsed_xml).build_housing_request)
      housing_request.is_active =  true
      housing_request.form_status="invalid"
      main_applicant.assign_attributes(Hermes::BoClient::Mappings::ApplicantReverseMap.new(parsed_xml, true).build_applicant)
      if main_applicant.save
        housing_request.owned_housing_details = main_applicant.owned_housing_details
        housing_request.actual_housing_details = main_applicant.actual_housing_details
      end
      xml_coapps = parsed_xml["interfaceNuu"]["demande"]["demandeLogement"]["personnePhysique"]["listeCotitulaire"]
      if xml_coapps
        xml_coapps = xml_coapps.is_a?(Array) ? xml_coapps : [xml_coapps]
        coapplicants = []
        xml_coapps.each_with_index do |xml_coapp, i|
          coapp_params = Hermes::BoClient::Mappings::ApplicantReverseMap.new(parsed_xml, false, i).build_applicant.merge(user_id: user_id)
          if coapplicant = Hermes::Applicant.create(coapp_params)
            coapplicants << { rel_type: xml_coapp["cotitulaire"]["lienDemandeur"]["code"],  id: coapplicant.id.to_s }
          end
        end
        housing_request.coapplicants = coapplicants
      end 
      
      if housing_request.save
        self.housing_request_id = housing_request.id.to_s
        self.done = true
        save
        if Rails.env != 'test'
          so = Hermes::BoClient::AddRequestTask.new(housing_request_id: housing_request.id.to_s, nud_id: nud.id.to_s)
          xml = so.to_xml
          housing_request.set(viewed_by_colaborator_offre_logement: false, last_xml_flux_30: xml, updated_at: DateTime.current)
          so.execute
        end

        return housing_request
      else
        return nil
      end
    end
  end
end
