require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  include Rack::Test::Methods
  
  describe Api::Dmo::NotificationCenter::SubscriptionsController, :type => :api do
    
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper
    let!(:user) { create(:user) }
    let!(:applicant) { create(:applicant, notifications_count: 5) }
    let!(:new_key) { 'testing.key' }

    describe 'Actions' do
      before :each do
        authenticated_header(request, user)
      end

      it 'return subscription' do
        get :show, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id

        expect(response).to have_http_status(200)
        expect(json['data']['id']).to eq(applicant.subscription.id.to_s)
      end

      it 'should subscribe to a new key' do
        put :subscribe, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id, key: new_key

        expect(response).to have_http_status(200)
        expect(applicant.subscription.subscribed_to_email?(new_key)).to be(true)
      end

      it 'should unsubscribe to an existing key' do
        applicant.subscription.subscribe_to_email(new_key)
        put :unsubscribe, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id, key: new_key

        expect(response).to have_http_status(200)
        expect(applicant.subscription.subscribed_to_email?(new_key)).to be(false)
      end

      it 'should return error if provider does not exist' do
        put :unsubscribe, targetable_type: 'Hermes::Applicant', id: applicant.id, subscription_id: applicant.subscription.id, key: new_key, provider: :hello_word

        expect(response).to have_http_status(404)
      end
    end
  end

  # Swagger
  require_relative '../../../swagger_helper'

  describe Hermes::Api::Dmo::NotificationCenter::SubscriptionsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_SUBSCRIPTIONS = 'API_DMO_NOTIFICATIONS_SUBSCRIPTIONS'
    DESCRIPTION_HTML = "Chaque 'notifiable' ici 'Applicant' possède un subscription_id, il est possible de récupérer celui-ci sur chacune des notifications ou dans la réponse du modèle notifiable."

    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:admin) {create(:administrator)}
    
    context 'NotificationCenter' do
      path '/api/dmo/applicants/{applicant_id}/subscriptions' do
        get 'See all available subbscription keys' do
          tags TAGS_API_DMO_SUBSCRIPTIONS
          produces 'application/json'
          description DESCRIPTION_HTML
          
          parameter name: :applicant_id, in: :path, type: :string, required: true
          security [Bearer: {}]
          
          response '200', 'keys found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:applicant) { create(:applicant) }
            let!(:applicant_id) { applicant.id }
            
            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { ::NotificationCenter::Subscription.keys(applicant) }
            end
          end
        end
      end

      path '/api/dmo/applicants/{applicant_id}/subscriptions/{subscription_id}' do
        get 'Retrieves a subscription' do
          tags TAGS_API_DMO_SUBSCRIPTIONS
          produces 'application/json'
          description DESCRIPTION_HTML
          
          parameter name: :applicant_id, in: :path, type: :string, required: true
          parameter name: :subscription_id, in: :path, type: :string, required: true
          security [Bearer: {}]
          
          response '200', 'subscription found' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:applicant) { create(:applicant) }
            let!(:subscription_id) { create(:notifications_subscription, subscriber: applicant).id }
            let!(:applicant_id) { applicant.id }
            
            include_context 'with integration test'
            include_context 'valid jsonapi'
          end
        end
      end

      path '/api/dmo/applicants/{applicant_id}/subscriptions/{subscription_id}/subscribe' do
        put 'Subscribe to a key' do
          tags TAGS_API_DMO_SUBSCRIPTIONS
          produces 'application/json'
          description DESCRIPTION_HTML
          
          parameter name: :applicant_id, in: :path, type: :string, required: true
          parameter name: :subscription_id, in: :path, type: :string,  required: true
          parameter name: :key, in: :query, type: :string, enum: ['housing_request.status.change', 'housing_offer.new.eligible'], required: true
          parameter name: :provider, in: :query, type: :string, enum: [:email], required: false, default: :email
          security [Bearer: {}]

          response '200', 'key subscribed' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:applicant) { create(:applicant) }
            let!(:subscription_id) { create(:notifications_subscription, subscriber: applicant).id }
            let!(:applicant_id) { applicant.id }
            let!(:key) { 'testing.key' }
            
            include_context 'with integration test'
            include_context 'valid jsonapi'
          end
        end
      end

      path '/api/dmo/applicants/{applicant_id}/subscriptions/{subscription_id}/unsubscribe' do
        put 'Unsubscribe to a key' do
          tags TAGS_API_DMO_SUBSCRIPTIONS
          produces 'application/json'
          description DESCRIPTION_HTML
          
          parameter name: :applicant_id, in: :path, type: :string, required: true
          parameter name: :subscription_id, in: :path, type: :string,  required: true
          parameter name: :key, in: :query, type: :string, enum: ['housing_request.status.change', 'housing_offer.new.eligible'], required: true
          parameter name: :provider, in: :query, type: :string, enum: [:email], required: false, default: :email
          security [Bearer: {}]

          response '200', 'new key subscribed' do
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:applicant) { create(:applicant) }
            let!(:subscription_id) { create(:notifications_subscription, subscriber: applicant).id }
            let!(:applicant_id) { applicant.id }
            let!(:key) { 'testing.key' }
            
            include_context 'with integration test'
            include_context 'valid jsonapi'
          end
        end
      end
    end
  end
end